# wom.py - An asynchronous wrapper for the Wise Old Man API.
# Copyright (c) 2023-present Jonxslays
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""This module contains the models used to represent data returned
by the WOM API.

Enums related to specific services are also housed in the module.

!!! tip

    Most of the models here you won't create, but a few you will. Those
    will be documented as such.
"""

from __future__ import annotations

__all__ = (
    "Achievement",
    "AchievementMeasure",
    "AchievementProgress",
    "Activity",
    "ActivityGains",
    "ActivityLeader",
    "BaseModel",
    "Boss",
    "BossGains",
    "BossLeader",
    "Country",
    "Competition",
    "CompetitionDetail",
    "CompetitionHistoryDataPoint",
    "CompetitionParticipationDetail",
    "CompetitionParticipation",
    "CompetitionProgress",
    "CompetitionStatus",
    "CompetitionType",
    "CompetitionWithParticipations",
    "ComputedGains",
    "ComputedMetric",
    "ComputedMetricLeader",
    "DeltaLeaderboardEntry",
    "DeniedNameChangeReviewContext",
    "Gains",
    "GroupDetail",
    "Group",
    "GroupActivity",
    "GroupActivityType",
    "GroupHiscoresActivityItem",
    "GroupHiscoresBossItem",
    "GroupHiscoresComputedMetricItem",
    "GroupHiscoresEntry",
    "GroupHiscoresSkillItem",
    "GroupMemberFragment",
    "GroupMemberGains",
    "GroupMembership",
    "GroupRole",
    "GroupStatistics",
    "HttpErrorResponse",
    "HttpSuccessResponse",
    "Membership",
    "MetricLeaders",
    "NameChange",
    "NameChangeReviewContext",
    "NameChangeReviewReason",
    "NameChangeStatus",
    "Participation",
    "PlayerAchievementProgress",
    "PlayerBuild",
    "PlayerCompetitionStanding",
    "PlayerMembership",
    "Player",
    "PlayerDetail",
    "PlayerGains",
    "PlayerGainsData",
    "PlayerParticipation",
    "PlayerStatus",
    "PlayerType",
    "Record",
    "RecordLeaderboardEntry",
    "Skill",
    "SkillGains",
    "SkillLeader",
    "SkippedNameChangeReviewContext",
    "SnapshotData",
    "Snapshot",
    "SnapshotTimelineEntry",
    "Team",
    "Top5ProgressResult",
)

from .base import *
from .competitions import *
from .deltas import *
from .groups import *
from .http import *
from .names import *
from .players import *
from .records import *



# wom.py - An asynchronous wrapper for the Wise Old Man API.
# Copyright (c) 2023-present Jonxslays
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""This module contains various exceptions used by the project."""

from __future__ import annotations

import typing as t

if t.TYPE_CHECKING:
    from wom.serializer import SerializerT

__all__ = ("FailedToDeserialize", "UnwrapError", "WomError")

T = t.TypeVar("T")


class WomError(Exception):
    """The base error all wom errors inherit from."""

    __slots__ = ()


class UnwrapError(WomError):
    """Raised when calling [`unwrap()`][wom.Result.unwrap] or
    [`unwrap_err()`][wom.Result.unwrap_err] incorrectly.

    Args:
        message: The error message.
    """

    __slots__ = ()

    def __init__(self, message: str) -> None:
        super().__init__(f"Unwrap failed: {message}")


class FailedToDeserialize(WomError, t.Generic[T]):
    """Raised when a response from the WOM api fails to deserialize.

    Args:
        method: The serializer method that was being called.
        exc: The exception that occurred.
    """

    def __init__(self, method: SerializerT[T], exc: Exception) -> None:
        super().__init__(f"Serializer call to `{method.__name__}` failed\n -> {exc}")



# wom.py - An asynchronous wrapper for the Wise Old Man API.
# Copyright (c) 2023-present Jonxslays
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from __future__ import annotations

from datetime import datetime

import attrs

from wom import enums

from ..base import BaseModel
from ..players import Player

__all__ = ("Record", "RecordLeaderboardEntry")


@attrs.define(init=False)
class Record(BaseModel):
    """Represents a record held by a player."""

    id: int
    """The unique ID for this record."""

    player_id: int
    """The player ID associated with this record."""

    period: enums.Period
    """The [`Period`][wom.Period] over which this record was achieved."""

    metric: enums.Metric
    """The [`Metric`][wom.Metric] measured in this record."""

    value: int
    """The records gained value."""

    updated_at: datetime
    """The records creation/modification date."""


@attrs.define(init=False)
class RecordLeaderboardEntry(BaseModel):
    """Represents a player's record leaderboard entry."""

    player: Player
    """The [`Player`][wom.Player] holding this leaderboard entry."""

    record: Record
    """The [`Record`][wom.Record] tied to this leaderboard entry."""



# wom.py - An asynchronous wrapper for the Wise Old Man API.
# Copyright (c) 2023-present Jonxslays
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""This module contains the services used to interact with different
portions of the WOM API.
"""

from __future__ import annotations

__all__ = (
    "BaseService",
    "CompetitionService",
    "DeltaService",
    "EfficiencyService",
    "GroupService",
    "HttpService",
    "NameChangeService",
    "PlayerService",
    "RecordService",
)

from .base import *
from .competitions import *
from .deltas import *
from .efficiency import *
from .groups import *
from .http import *
from .names import *
from .players import *
from .records import *
